require "isbn-test"
local ISBN = require "isbn"

module("test.albania", lunit.testcase, package.seeall)

local function test_isbn (isbn, input, expected)
    assert_table(isbn)
    assert_isa(isbn, ISBN)
    is(input, isbn:isbn(), "isbn() method")
    is(expected.group_code, isbn:group_code(), "has right group code")
    is(expected.publisher, isbn:publisher_code(), "has right publisher")
    is(expected.group_name, isbn:group_name(), "has right group name")
    is(expected.article_code, isbn:article_code(), "has right article code" )
    is(expected.checksum, isbn:checksum(), "has right checksum" )
    is(expected.with_hyphens, tostring(isbn), "stringify metamethod")
end

function test_module_metadata ()
    assert_function(ISBN.new)
    assert_equal("isbn", ISBN._NAME, "_NAME")
end

local ALBANIAN_EXPECTED = {
    with_hyphens = "99927-0-157-9",
    group_name   = "Albania",
    group_code   = "99927",
    publisher    = "0",
    article_code = "157",
    checksum     = "9",
}

function test_albanian_call_constructor ()
    local isbn, err = ISBN("9992701579")
    assert_nil(err)
    test_isbn(isbn, "9992701579", ALBANIAN_EXPECTED)
end

function test_albanian_old_new_method ()
    local isbn, err = ISBN:new("9992701579")
    assert_nil(err)
    test_isbn(isbn, "9992701579", ALBANIAN_EXPECTED)
end

function test_eq_isbn10 ()
    local isbn10 = ISBN("9992701579")
    assert_true(isbn10:eq("999-270 1 5 7-9"))
    assert_true(isbn10:eq(ISBN("9992701579")))
    assert_true(isbn10:eq("978-99927-0-157-7"))
    assert_true(isbn10:eq(ISBN("978-99927-0-157-7")))

    assert_false(isbn10:eq("000761263X"))
    assert_false(isbn10:eq(ISBN("000761263X")))
    assert_false(isbn10:eq("9780900083181"))
    assert_false(isbn10:eq(ISBN("9780900083181")))

    assert_error(function () isbn10:eq("invalid") end)
end

function test_eq_isbn13 ()
    local isbn13 = ISBN("978-99927-0-157-7")
    assert_true(isbn13:eq("999-270 1 5 7-9"))
    assert_true(isbn13:eq(ISBN("9992701579")))
    assert_true(isbn13:eq("978-99927-0-157-7"))
    assert_true(isbn13:eq(ISBN("978-99927-0-157-7")))

    assert_false(isbn13:eq("000761263X"))
    assert_false(isbn13:eq(ISBN("000761263X")))
    assert_false(isbn13:eq("9780900083181"))
    assert_false(isbn13:eq(ISBN("9780900083181")))
end

function test_eq_operator ()
    assert_true(ISBN("9992701579")     == ISBN("9992701579"))
    assert_true(ISBN("9789992701577")  == ISBN("9992701579"))
    assert_true(ISBN("9992701579")     == ISBN("9789992701577"))
    assert_true(ISBN("9789992701577")  == ISBN("9789992701577"))

    assert_false(ISBN("9992701579")    == ISBN("000761263X"))
    assert_false(ISBN("9789992701577") == ISBN("000761263X"))
    assert_false(ISBN("9992701579")    == ISBN("9780900083181"))
    assert_false(ISBN("9789992701577") == ISBN("9780900083181"))

    assert_true(ISBN("9992701579")    ~= ISBN("000761263X"))
    assert_true(ISBN("9789992701577") ~= ISBN("000761263X"))
    assert_true(ISBN("9992701579")    ~= ISBN("9780900083181"))
    assert_true(ISBN("9789992701577") ~= ISBN("9780900083181"))
end

-- vi:ts=4 sw=4 expandtab
